//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  Gainsboro
#property indicator_color2  HotPink
#property indicator_width1  3
#property indicator_width2  3

//
//
//
//
//

extern int    FilterPeriod    = 100;
extern int    FilterPrice     = PRICE_CLOSE;
extern int    FilterType      = 0;
extern double FrequencyCutoff = 0.01;
extern bool   HighPass        = false;
extern string UniqueID        = "Sync filters - windows";
extern int    Shift           = 50;

double filh[];
double fill[];
double coeffs[];
int    half;
string shortName;

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,filh); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,fill);
         
      //
      //
      //
      //
      //

      FrequencyCutoff = MathMax(MathMin(FrequencyCutoff,0.4999999),0.00000001);
      FilterType      = MathMax(MathMin(FilterType,13),0);
      FilterPeriod    = MathMax(FilterPeriod,1);
                      if (MathMod(FilterPeriod,2)==0) FilterPeriod += 1; half = MathFloor(FilterPeriod/2.0);
                      if (HighPass)
                            { fillCoeffs(coeffs,FrequencyCutoff,FilterPeriod,FilterType,-1); coeffs[half] = coeffs[half]+1.0; }
                      else  { fillCoeffs(coeffs,FrequencyCutoff,FilterPeriod,FilterType,+1); }
      shortName = UniqueID+" : "+WindowName(FilterType);
      IndicatorShortName(shortName);
   return(0);
}

//
//
//
//
//

int deinit()
{
   string lookFor = UniqueID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i); if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted(),half = MathFloor(FilterPeriod/2.0);
      if (counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(MathMax(Bars-counted_bars,half),Bars-1);


         //
         //
         //
         //
         //

         double max = 0; for (i=0; i<FilterPeriod ; i++) { fill[i+Shift] = coeffs[i]; filh[i+Shift] = fill[i+Shift]; max = MathMax(max,fill[i+Shift]); }
         
         //
         //
         //
         //
         //
         
         int    window = WindowFind(shortName);
         string name   = UniqueID+":text";                   
            if (ObjectFind(name) == -1)
                ObjectCreate(name,OBJ_TEXT,window,0,0);
                   ObjectSet(name,OBJPROP_TIME1,Time[0+Shift]+15*Period()*60);
                   ObjectSet(name,OBJPROP_PRICE1,max/2);
                   ObjectSet(name,OBJPROP_COLOR,Black);
                   ObjectSetText(name,"type "+FilterType+" - "+WindowName(FilterType),10,"Courier new");
                   
   SetIndexDrawBegin(0,Bars-FilterPeriod-Shift+1);
   SetIndexDrawBegin(1,Bars-FilterPeriod-Shift+1);
   return(0);
}


//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

#define Pi 3.141592653589793238462643383279502884197169399375105820974944592

string names[] = {"Hamming","Hanning","Blackman","Blackman Harris","Blackman Nutall","Nutall","Bartlet zero end points","Bartlet-Hann","Hann","Sine","Lanczos","Flat top","No window"};
string WindowName(int& type)
{
   int max = ArraySize(names); type = MathMax(MathMin(type,max-1),0);
   return (names[type]);
}

//
//
//
//
//

#define Pi 3.141592653589793238462643383279502884197169399375105820974944592

void fillCoeffs(double& coeffs[], double frequencyCutoff, double filterPeriod, int filterType, double multiplier=1)
{
   ArrayResize(coeffs,filterPeriod);
   
   //
   //
   //
   //
   //
   
   int N = filterPeriod-1; double sum = 0;  
   for (int n=0; n<filterPeriod; n++)
   {
      double div = (n-N/2.0);
         if (div==0) coeffs[n]  = 2.0*Pi*frequencyCutoff;
         else        coeffs[n]  = MathSin(2.0*Pi*frequencyCutoff*div)/div;

      //
      //
      //
      //
      //

      switch(filterType)
      {
         case 0 :  coeffs[n] *= (0.54-0.46*MathCos(2*Pi*n/N));                                                                     break; // Hamming
         case 1 :  coeffs[n] *= (0.50-0.50*MathCos(2*Pi*n/N));                                                                     break; // Hanning
         case 2 :  coeffs[n] *= (0.42-0.50*MathCos(2*Pi*n/N)+0.08*MathCos(4*Pi*n/N));                                              break; // Blackman
         case 3 :  coeffs[n] *= (0.35875-0.48829*MathCos(2*Pi*n/N)+0.14128*MathCos(4*Pi*n/N)+0.01168*MathCos(6*Pi*n/N));           break; // Blackman Harris
         case 4 :  coeffs[n] *= (0.3635819-0.4891775*MathCos(2*Pi*n/N)+0.1365995*MathCos(4*Pi*n/N)+0.0106411*MathCos(6*Pi*n/N));   break; // Blackman Nutall
         case 5 :  coeffs[n] *= (0.355768-0.487396*MathCos(2*Pi*n/N)+0.144232*MathCos(4*Pi*n/N)+0.012604*MathCos(6*Pi*n/N));       break; // Nutall
         case 6 :  coeffs[n] *= (2.0/N*(N/2.0-MathAbs(n-N/2)));                                                                    break; // Bartlet zero end points
         case 7 :  coeffs[n] *= (0.62-0.48*MathAbs(n/N-0.5)*0.38*MathCos(2.0*Pi*n/N));                                             break; // Bartlet-Hann
         case 8 :  coeffs[n] *= (0.50*(1.0-MathCos(2.0*Pi*n/N)));                                                                  break; // Hann
         case 9 :  coeffs[n] *= (MathSin(Pi*n/N));                                                                                 break; // Sine
         case 10:  
                double ttx = ((n/N)-1.0);
                   if (ttx==0)
                         coeffs[n] *= 0;
                   else  coeffs[n] *= (MathSin(ttx)/(ttx));                                                                        break; // Lanczos
         case 11:  coeffs[n] *= (1-1.93*MathCos(2*Pi*n/N)+1.29*MathCos(4*Pi*n/N)+0.388*MathCos(6*Pi*n/N)+0.388*MathCos(8*Pi*n/N)); break; // Flat top
      }
      
      //
      //
      //
      //
      //
      
      sum += coeffs[n];
   }
   for (int k=0; k<filterPeriod; k++) { coeffs[k] = coeffs[k]/sum; coeffs[k] = coeffs[k]*multiplier; }
}